/* This file is correlate.cc
   Function declaration of correlate which accumulates terms related to SRO_pair,
   <P(x,i),P(y,j)>, where x and y denotes any 2 sites, i and j denotes atomic types
   By Teck Leong Tan
*/

#include "lattice.h"

int convert(int noccu1, int noccu2, int num_type);//Return correct species index for <P(x,i), P(y,j)>, i<j

void Lattice::correlate()
{
  //SRO pair=============================================================================

  //Note: <P(x,i), P(y,j)> is averaged over sites with constant x minus y . 
  if (pair_option>0)//Average over x-y=constant
  {
    //Loop over cell
    for (int ijk=0; ijk<nsize;ijk++)
    {
      //Loop over no. of sublattices
      for (int ll=0; ll<num_sublat;ll++)
      {
        
	      for (int nei=0;nei< noccupancy[ijk].site[ll].cor_pair.size();nei++)
	      {
	        int ijk2=noccupancy[ijk].site[ll].cor_pair[nei]->ijk;
	        int ll2=noccupancy[ijk].site[ll].cor_pair[nei]->l;
	        int noccu1,noccu2;
          //Only if noccu1 < noccu2 will terms be accumulated
	        if (noccupancy[ijk].site[ll].atomtype<noccupancy[ijk2].site[ll2].atomtype)
	        {
            noccu1=noccupancy[ijk].site[ll].atomtype;
            noccu2=noccupancy[ijk2].site[ll2].atomtype;

            //Count no. of instances for each <P(x,i),P(y,j)> term
            noccupancy[0].site[ll].cor_pair[nei]->num_instances[convert(noccu1,noccu2,num_type)]+=1;
            int temp=convert(noccu1,noccu2,num_type);
            
          }
        }
      }//end for (int l=0; l<num_sublat;l++)
    }// end for (int ijk=0; ijk<nsize;ijk++)
  }//end if
  
  else //degenerate clusters are accumulated separately. (Not used yet)
  {
    //Loop over cell
    for (int ijk=0; ijk<nsize;ijk++)
    {
      //Loop over no. of sublattices
      for (int ll=0; ll<num_sublat;ll++)
      {
	      for (int nei=0;nei< noccupancy[ijk].site[ll].cor_pair.size();nei++)
	      {
	        int ijk2=noccupancy[ijk].site[ll].cor_pair[nei]->ijk;
          int ll2=noccupancy[ijk].site[ll].cor_pair[nei]->l;
          int noccu1,noccu2;
          if (noccupancy[ijk].site[ll].atomtype<noccupancy[ijk2].site[ll2].atomtype)
          {
	          noccu1=noccupancy[ijk].site[ll].atomtype;
            noccu2=noccupancy[ijk2].site[ll2].atomtype;
            noccupancy[ijk].site[ll].cor_pair[nei]->num_instances[convert(noccu1,noccu2,num_type)]+=1;
          }
          else
          {
	          noccu2=noccupancy[ijk].site[ll].atomtype;
            noccu1=noccupancy[ijk2].site[ll2].atomtype;
            noccupancy[ijk].site[ll].cor_pair[nei]->num_instances[convert(noccu1,noccu2,num_type)]+=1;
          }
        }
      }//end for (int l=0; l<num_sublat;l++)
    }// end for (int ijk=0; ijk<nsize;ijk++) 
  }//end else
  //End of SRO pair option================================================================


  //SRO user define option================================================================
  //!!!!Warning this part of the code is not tested!!!!;

  if (user_def_option==1)// Degenerate clusters are averaged over 
  {
    //Spatial option. Average over degenerate sites---------------------------------------

    //Loop over num of user defined clusters
    for (int nei =0;nei<user_def_param_spa.size();nei++)//Need to ensure that the no. of clusters is correct
    {
      int templ=user_def_param_spa[nei]->coord[0].sublat;//Sublattic of interest
      int temp_size=noccupancy[0].site[templ].cor_def_spa[nei]->ijk.size();
      //Loop over cell
      for (int ijk=0; ijk<nsize;ijk++)
      {
        vector<int> temp_indices;
        temp_indices.assign(temp_size,0);
        for (int i=0;i<temp_indices.size();i++)
        {
	        int ijk2=noccupancy[ijk].site[templ].cor_def_spa[nei]->ijk[i];
          int ll2=noccupancy[ijk].site[templ].cor_def_spa[nei]->l[i];
          if (noccupancy[ijk2].site[ll2].atomtype>0)
	        {
            temp_indices[i]=noccupancy[ijk2].site[ll2].atomtype; 
          }
          else
          {
            temp_indices.assign(1,0);
            break;
          }
          
        }
        if (temp_indices[0]==0) {continue;}
        noccupancy[0].site[templ].cor_def_spa[nei]->\
        num_instances[(linear_index(temp_size,temp_indices))]+=1;
      }
    }//End of spatial option---------------------------------------------------------------


    //Fixed option. Site is unique, no spatial averag (NOT in USE!)-------------------------

    for (int nei =0;nei<user_def_param_fix.size();nei++)//Need to ensure that the no. of clusters is correct
    {
      int templ=user_def_param_fix[nei]->coord[0].sublat;//Sublattic of interest
      int temp_size=noccupancy[0].site[templ].cor_def_fix[nei]->ijk.size();
      int temp_ijk;
      //int cartesian[ndim];
      temp_ijk=user_def_param_fix[nei]->first_cell;
 
      vector<int> temp_indices;//Stores the atomic species occupying the cluster
      temp_indices.assign(temp_size,0);
      for (int i=0;i<temp_indices.size();i++)
      {
        int ijk2=noccupancy[temp_ijk].site[templ].cor_def_fix[nei]->ijk[i];
        int ll2=noccupancy[temp_ijk].site[templ].cor_def_fix[nei]->l[i];
        if (noccupancy[ijk2].site[ll2].atomtype>0)
        {
            temp_indices[i]=noccupancy[ijk2].site[ll2].atomtype; 
        }
        else
        {
            temp_indices.assign(1,0);
            break;
        }
          
      }
      if (temp_indices[0]==0) {continue;}
      noccupancy[temp_ijk].site[templ].cor_def_fix[nei]->\
      num_instances[(linear_index(temp_size,temp_indices))]+=1;
    }
  }//End SRO user define option==============================================================

}

int convert(int noccu1, int noccu2, int num_type)
{ int index=0;
  for (int i=0;i<=noccu1;i++)
  {
    index +=(i>0)*(num_type-i+1);
  }
  index+=(noccu2-noccu1);
  return (index-1);
}

//EOF






