//This file is lattice.cc
//Contains constructors, destructors & some funcs of the Lattice class
//By Teck Leong Tan

#include "lattice.h"

//Default constructor
Lattice::Lattice()
{ 
  //Initialise all variables:
  ndim=1;
  nsize=1;
  num_type=1; 
  nrep.assign(ndim,0);
  factor.assign(ndim,1);
  canonical='c';
  strcpy(inconf0,"inconfig0.lat");
  //Initialise temperature related variables
  Tinit=0; Tfinal=0; Tincrement=0; Tsub=1;
  maxbody=0; 
  randseed=44123847;
  write_freq=1;
  equil_pass=0;
  numpass=1;
  restart=0;
  correlation_option=0;
  pair_option=0;
  pair_cutoff=1;
  user_def_option=0;
  LRO_option=0;
  num_runs=1;
  sinv_option=0;
  //lat_vec_scale=1;
  //inlatvec_file="inlatvec.txt";
  sinv_dir.clear();
  stdata_dir.clear();
  fracnn2=0.1;
  outlevel = "low";
}

//Destructor
Lattice::~Lattice()
{
  noccupancy.clear();
  //conc.clear();
  //amount.clear();
  //delete inconf0;
  //delete &E_struct;
  for (int ijk=0; ijk<noccupancy.size();ijk++)
  {
    for (int l=0; l<noccupancy[ijk].site.size();l++)
    {
      for (int i=0;i<noccupancy[ijk].site[l].neighbors.size();i++)
      {
	delete noccupancy[ijk].site[l].neighbors[i];
      }
      for (int i=0;i<noccupancy[ijk].site[l].cor_pair.size();i++)
      {
	delete noccupancy[ijk].site[l].cor_pair[i];
      }
      for (int i=0;i<noccupancy[ijk].site[l].cor_def_spa.size();i++)
      {
	delete noccupancy[ijk].site[l].cor_def_spa[i];
      }
      for (int i=0;i<noccupancy[ijk].site[l].cor_def_fix.size();i++)
      {
	delete noccupancy[ijk].site[l].cor_def_fix[i];
      }
    }
  }
  /*for (int i=0; i<sinv_filenames.size();i++)
  {
    delete sinv_filenames[i];
    }*/ 
 // delete [] key_array;
}

//Member function of Lattice class
void Lattice::read_clus()
{
  fstream clusfile;
  int ss, nn, ff, dd;
  Sublattice vsublattice; //a vacant variable for assignment purposes 
  Nbody vnbody; //a vacant variable for assignment purposes
  string filename;
  if (sinv_option==1 && !sinv_dir.empty()) {filename=sinv_dir+"/"+inclusfile;}
  else {filename=inclusfile;}
  clusfile.open(filename.c_str(), ios::in);
  if (!clusfile)
  {
    cout<<"File: "<<filename<<" is not opened!"<<endl;
    exit(1);
  }  
  clustree.sublat.assign(num_sublat, vsublattice);//assign num_sublat number of vacant variable vsublattice
  //cout<<"lattice.cc line 65, size of sublat= "<<clustree.sublat.size()<<endl;
  
  //Assignment for zero-body cluster
  //Loop over sublattice
  for (int l=0; l<num_sublat;l++)
  {
    clustree.sublat[l].nbody.assign(maxbody+1, vnbody);
    clustree.sublat[l].nbody[0].nclus.push_back((new Ncluster));
    clustree.sublat[l].nbody[0].nclus[0]->nsym.push_back((new Nsymmetry));
    clustree.sublat[l].nbody[0].nclus[0]->nsym[0]->ref.push_back(0);
  }
  //cout<<"lattice.cc, line75"<<endl;
  //Read in the cluster tree structure---------------------------------
  while (1)
  {
    clusfile.ignore(256, '\n');//Ignore 1st line of clus.in
    clusfile>>ss;//Read in sublattice index
    //cout<<"line 90: ss= "<<ss<<endl;
    if (clusfile.eof()) {break;} //Break when EOF
    clusfile>>nn>>ff>>dd;//Read in nbody, cluster, symmetry indices.
    if (nn>maxbody && maxbody!=-1)
    {
      cout<<"Warning in lattice.cc: Cluster size is up to "<<maxbody<<"-body cluster";
      cout<<endl;
      break;
    }
    if (clustree.sublat[0].nbody[nn].all_clus.size()<ff)
    //Note ff starts from 1
    { 
      for (int i=0; i<ff-clustree.sublat[0].nbody[nn].all_clus.size();i++)
      {
        vector<int> dummy;
        //Keeps track of the total no.f- indices for given nn-body clus.
        clustree.sublat[0].nbody[nn].all_clus.push_back(dummy);
        clustree.sublat[0].nbody[nn].f_degen.push_back(1);
      }	
    }
    else{clustree.sublat[0].nbody[nn].f_degen[ff-1]+=1;}
 
    if (clustree.sublat[ss].nbody[nn].nclus.empty()|| \
        clustree.sublat[ss].nbody[nn].nclus.back()->f_index<ff-1)
    {	
      clustree.sublat[ss].nbody[nn].nclus.push_back((new Ncluster));
      clustree.sublat[ss].nbody[nn].nclus.back()->f_index=ff-1;
    }  
    clustree.sublat[ss].nbody[nn].nclus.back()->nsym.push_back((new Nsymmetry));

    //Loop over nbody
    for (int i=0; i<nn;i++)
    {
      int index;
      clusfile>>index;//Read in neighbor references
      clustree.sublat[ss].nbody[nn].nclus[ff-1]->nsym.back()->ref.push_back(index);     
      if (index>=neighborlist[ss].size())
      {
        cerr<<"Error in lattice.cc: No such neighbor index: "<<index<<endl;
        exit(1);
      }
      //cout<<"index= "<<index<<endl;
    }//end for (int i=0; i<nn;i++)
    //cout<<"line 128: loop done"<<endl;
  }//end while (1)
 
  if (clustree.sublat[0].nbody[maxbody].nclus.empty())
  {
    cout<<"Error in lattice.cc: No. of interacting bodies in cluster file";
    cout<<" does not match no. of interacting bodies in input file"<<endl;
  }
  //cout<<"size is "<<clustree.sublat[0].nbody[maxbody].nclus.size()<<endl; 
  //cout<<"size is "<<clustree.sublat[0].nbody[2].nclus.size()<<endl;
  //End of reading in cluster tree structure------------------------------- 
    
  clusfile.close();
  //cout<<"lattice.cc , end of read_clus()"<<endl;
  //Print to file cluster information. For checking purpose-----------
  fstream outclus;
  outclus.open(outclusfile, ios::out);
  if (!outclus)
  {
     cout<<"File: "<<outclusfile<<" is not opened!"<<endl;
     exit(1);
  }
  outclus<<"# Sublat, nbody, nclus, nsym, ref"<<endl;
  for (int l=0;l<num_sublat;l++)
  {
   //outclus<<" num_sublat= "<<num_sublat;
   for (int n=1;n<=maxbody;n++)
   {
     //outclus<<" maxbody= "<<maxbody;
     for (int f=0;f<clustree.sublat[l].nbody[n].nclus.size();f++)
     {
       //outclus<<" nclus.size()= "<<clustree.sublat[l].nbody[n].nclus.size();
       for (int d=0;d<clustree.sublat[l].nbody[n].nclus[f]->nsym.size();d++)
       {
         //outclus<<" nsym.size()= "<<clustree.sublat[l].nbody[n].nclus[f]->nsym.size();
         outclus<<l<<" "<<n<<" "<<f<<" "<<d<<"  ";
         for (int r=0;r<clustree.sublat[l].nbody[n].nclus[f]->nsym[d]->
 ref.size();r++)
         {
           //outclus<<"ref.size()= "<<clustree.sublat[l].nbody[n].nclus[f]->nsym[d]->ref.size();
  	   outclus<<clustree.sublat[l].nbody[n].nclus[f]->nsym[d]->ref[
 r]<<" ";
         }//end for (int r=0; )
         outclus<<endl;
       }//end for (int d=0;)
     }//end for (int f=0;)
    }//end for (int n=1;)
  }//end for (int l=0;)
  outclus.close();
  //End of print out to outclus;---------------------------------------
  cout<<"Done with inclus.txt"<<endl;
}

//Member function of Lattice class
void Lattice::read_neighbor()
{
  fstream neifile;
  int sub, nei;
  int ll;
  int rposition[ndim];
  string filename;
  if (sinv_option==1 && !sinv_dir.empty()) {filename=sinv_dir+"/"+inneighborfile;}
  else {filename=inneighborfile;}
  neifile.open(filename.c_str(), ios::in);
  if (!neifile)
  {
    cout<<"File: "<<filename<<" is not opened!"<<endl;
    exit(1);
  }

  //Reading in neighbor info.
  while (1)
  {
    neifile.ignore(256, '\n');//Ignore 1st line of in_neighbor.txt
    neifile>>sub;//Read in sublattice index
    if (neifile.eof()) {break;} //Break when EOF
    //Creating neighborlist
    if(neighborlist.size()<=sub)
    {
      vector<Nei_pos> dummy;
      neighborlist.push_back(dummy);
    }
    
    neifile>>nei;//Read in neighbor index
    Nei_pos dummy_nei_pos;
    neighborlist[sub].push_back(dummy_nei_pos);

    //Read relative position of neighbors
    //cout<<"relative position: "<<endl;
    for (int i= 0; i<ndim;i++)
    {
      neifile>>rposition[i];
      neighborlist[sub].back().rposition.push_back(rposition[i]);
      //cout<<rposition[i]<<" ";
    }
    neifile>>ll; //Read in neighbor's sublattice index
    neighborlist[sub].back().l=ll;
    //cout<<neighborlist[sub].back().l<<endl;
    //Loop over no. of cells
    for (int ijk=0; ijk<nsize;ijk++)
    { 
      int nei_cartesian[ndim];//neighbor's cartesian coordinates
      int nei_ijk;//neighbor's position represented by ijk
      int cartesian[ndim];//atom ijk's cartesian coordinate
      
      //Convert ijk to cartesian coordinates
      ijk_to_cartesian(ijk, cartesian);
      
      //Loop over ndim (no. of directions)
      /*for (int i=ndim-1; i>=0;i--)
      { 
        //Apply periodic boundary conditions---------------------------

	nei_cartesian[i]=cartesian[i]+rposition[i];
	
        //if out of range 
        if (nei_cartesian[i]<0)
        {
	  nei_cartesian[i]+=nrep[i];
	  //if neicartesian still < 0
 	  if (nei_cartesian[i]<0)
	  {
	    cout<<"Error: Increase lattice size or decrease interaction length"<<endl;
	    exit(1);
	  }
        }
        //if out of range          
        if (nei_cartesian[i]>=nrep[i])
        {
          nei_cartesian[i]-=nrep[i];
          //if neicartesian still >=nrep[i]
          if (nei_cartesian[i]>=nrep[i])
          {
            cout<<"Error: Increase lattice size or decrease interaction length"<<endl;
            exit(1);
          }
	  }//end if (nei_cartesian[i]>=nrep[i])

	  boundary (cartesian, nei_cartesian, rposition
	  //End of checking for boundary condition-----------------------
      //boundary (cartesian, nei_cartesian, rposition);
      }//end for (int i=ndim-1; i>=0;i--)*/
	
      boundary (cartesian, nei_cartesian, rposition);	
      nei_ijk=cartesian_to_ijk(nei_cartesian);
      //cout<<"line 243: nei_ijk= "<<nei_ijk<<endl;

      //Check if size of neighbor is smaller than neighbor index
      if ( noccupancy[ijk].site[sub].neighbors.size()<nei+1)
      {
        //'Smart' loop over (nei - no. of neighbors)
        //Even if nei is not sequential, neighbors will be stored correct
        for (int i=0; i<(nei+1-noccupancy[ijk].site[sub].neighbors.size());i++)
        {
          noccupancy[ijk].site[sub].neighbors.push_back((new NeighborO));
        }
      }//end if ( noccupancy[ijk].site[ll].neighbors.size()<nei+1)
      
      
      noccupancy[ijk].site[sub].neighbors[nei]->ijk=nei_ijk;
      noccupancy[ijk].site[sub].neighbors[nei]->l=ll;
      //cout<<"Size of .neighbors= "<<noccupancy[ijk].site[sub].neighbors.size()<<endl;
      //cout<<"Neighbors of atom ijk= "<<ijk<<" l= "<<ll;
      //cout<<" : "<<noccupancy[ijk].site[sub].neighbors[nei]->ijk<<endl;
    }//end for (int ijk=0; ijk<nsize;ijk++)
  }//end while(1)  
  neifile.close();
  //cout<<"lattice.cc line 217, end of read_clus()"<<endl;

  //cout<<"neifile closed"<<endl;
  //Print to file neighbor information. For checking purpose-----------
  fstream outnei;
  outnei.open(outneighborfile, ios::out);
  if (!outnei)
  {
    cout<<"File: "<<outneighborfile<<" is not opened!"<<endl;
    exit(1);
  }
  outnei<<"# Site ijk, site's sublattice, neighbor ijk, neighbor's sublat."<<endl;
  for (int ijk=0;ijk<nsize;ijk++)
  {
    for (int l=0;l<num_sublat;l++)
    {
      for (int nei=0;nei<noccupancy[ijk].site[l].neighbors.size();nei++)
      {
        outnei<<ijk<<" "<<l<<"  ";
        outnei<<noccupancy[ijk].site[l].neighbors[nei]->ijk<<" ";
        outnei<<noccupancy[ijk].site[l].neighbors[nei]->l<<endl;
      }//end for (int nei=0;)
    }//end for (int l=0;)
  }//end for (int ijk=0;)
  outnei.close();
  //End of print out to outnei;---------------------------------------

  //Print neighborlist
  fstream outlist;
  outlist.open("outlist.txt", ios::out);
  if (!outlist)
  {
    cout<<"File: "<<"outlist.txt"<<" is not opened!"<<endl;
    exit(1);
  }
  //cout<<"size of neighborlist= "<<neighborlist.size();
  //cout<<"size of neighborlist[0]= "<<neighborlist[0].size();
  for (int i=0; i<neighborlist.size();i++)
  {
      //outlist<<"i= "<<i<<" list size= "<<neighborlist[i].size()<<endl; 
    for (int j=0; j<neighborlist[i].size();j++)
    {
      outlist<<i<<" "<<j<<" ";
      for (int k=0;k<neighborlist[i][j].rposition.size();k++)
      {
        outlist<<neighborlist[i][j].rposition[k]<<" ";
      }
      outlist<<neighborlist[i][j].l;
      outlist<<endl;
    }
  }
  outlist.close(); 
}    
	
//Memerber func.
void Lattice::ijk_to_cartesian(int ijk, int cartesian[])
{
  for (int i=ndim-1;i>=0;i-- )
  {
    cartesian[i]=ijk/factor[i];
    ijk=ijk%factor[i];
  }
  //cout<<"lattice.cc line 226:";
  
}  

//Member func.
int Lattice::cartesian_to_ijk(int carteisan[])
{
  int ijk=0;
  for (int i=0; i<ndim;i++)
  {
    ijk+=(carteisan[i]*factor[i]);
  }
  return ijk;
}  

int Lattice::read_config(fstream& confin)
{   //read noccupancy[] from a file.
    //Loop over cell (ijk)
    for (int ijk=0; ijk<nsize; ijk++){
      //Loop over sublattice
      for (int l=0; l<num_sublat;l++){
	  confin >> noccupancy[ijk].site[l].atomtype;
          cout<<noccupancy[ijk].site[l].atomtype;
      }
      //if ((ijk+1)%nrep[0]==0) { config<<endl; }
      //if(ndim>1){ if ((ijk+1)%nrep[1]==0){ config<<endl; } }
    }//end for (int ijk=0; ijk<nsize; ijk++)
    return 0; //0 if success, not 0 if error.
}

void Lattice::boundary(int cartesian[],int nei_cartesian[], int cartesian_r[])
{
  //Loop over ndim (no. of directions)
  for (int i=ndim-1; i>=0;i--)
  { 
    //Apply periodic boundary conditions---------------------------

    nei_cartesian[i]=cartesian[i]+cartesian_r[i];

    //if out of range 
    if (nei_cartesian[i]<0)
    {
      nei_cartesian[i]+=nrep[i];
      //if neicartesian still < 0
      if (nei_cartesian[i]<0)
      {
	cout<<"Error: Increase lattice size or decrease interaction length"<<endl;
	exit(1);
      }
    }
    //if out of range          
    if (nei_cartesian[i]>=nrep[i])
    {
      nei_cartesian[i]-=nrep[i];
      //if neicartesian still >=nrep[i]
      if (nei_cartesian[i]>=nrep[i])
      {
	cout<<"Error: Increase lattice size or decrease interaction length"<<endl;
	exit(1);
      }
    }//end if (nei_cartesian[i]>=nrep[i])
    //End of checking for boundary condition-----------------------

  }//end for (int i=ndim-1; i>=0;i--)	
}

//EOF//
