//This file is metro_canon.cc
//Definition of a member function of class Lattice
//By Teck Leong Tan and Nikolai Zarkevich

#include "lattice.h"
//extern int ndim, num_type;
//extern int nrep[3];// No. of cell repetitions in each direction
//extern int nsize; //nsize is the no. of atoms

//Canonical (fixed-composition) Metropolis algorithm:
void Lattice::metro_canon(double T, int pass)
{
  double beta; //Temporary
  beta=1.0/T;
  double Probability=0;
  int random;
  double dou_random;
  int noccu, noccu_new;

  //double Eold, Enew; //error_check_only!

  //Loop over cell (ijk)
  for (int ijk=0; ijk<nsize;ijk++)
  {
    //Loop over sublattice
    for (int l=0; l<num_sublat;l++)
    {
      /* Algorithm here is the following:
	 site1 (ijk,l) is chosen sequentially;
	 site2 (ijk2,l2) is chosen randomly.
	 Attempt to swap atoms between site1 and site2:
	 accepted if exp(-dE/kT)<rnd; 0<rnd<1.
      */

      //choose site2:
      int ijk2, l2;
      ijk2=rand()%(nsize);
      l2=rand()%(num_sublat);

      //Eold=eEnergy();

      //change occupancy of site1:
      noccu=noccupancy[ijk].site[l].atomtype;
      noccu_new=noccupancy[ijk2].site[l2].atomtype;

      //Print to screen config---------------------------------------------    
      //cout<<"Attempt to swap site "<<ijk<<" lat "<<l<<" with site ";
      //cout<<ijk2<<" lat "<<l2<<endl;
      //if no. of passes >= no. of passes for equilibration
      /*if (pass>equil_pass)
      {
	cout<<"dE= "<<dE<<endl;
	cout<<"noccupancy: "<<endl;
        for (int ijk=0; ijk<nsize; ijk++)
        { //Loop over sublattice
          for (int l=0; l<num_sublat;l++)
          {
            cout<<" "<<noccupancy[ijk].site[l].atomtype;
          }
          for (int i=ndim-1;i>0;i--)
          {
            if ((ijk+1)%factor[i]==0) { cout<<endl; }
       
          }
	}//end for (int ijk=0; ijk<nsize; ijk++)
	}//end_if (pass>equil_pass)*/
      //End of print to screen-----------------------------------------------     

      if(noccu==noccu_new) continue; //No need to swap identical atoms.
    
      double dE;
      dE = denergy(ijk, l, noccu_new);//Calculates dE on site1.

      noccupancy[ijk].site[l].atomtype = noccu_new; //swap on site1.

      dE+= denergy(ijk2, l2, noccu);//Calculates dE on site2.
 
      //error_check:
      //noccupancy[ijk2].site[l2].atomtype=noccu;
      //Enew=eEnergy();
      //noccupancy[ijk2].site[l2].atomtype=noccu_new;
      //end_error_check

      //cout<<endl;
      //cout<<"dE= "<<dE<<" = "<<Enew-Eold<<" == "<<Enew<<" - "<<Eold<<endl;
      
      //Accept or reject?-----------------------------------------
      if( dE <= 0 )
      {
	//accept:
	//noccupancy[ijk].site[l].atomtype=noccu_new; //done_before.
	noccupancy[ijk2].site[l2].atomtype=noccu; //swap on site2.
	//cout<<"Move accepted with P=1"<<endl;
	//E_struct.dEacc=E_struct.dEacc+dE;
      }
      else
      {
	Probability=exp(-dE*beta);
	dou_random=(double)rand()/(double)RAND_MAX;//Produce rand. no. drom 0 to 1;
        //cout<<"RANDMAX= "<<RAND_MAX<<endl;  
        //cout<<"Raw random no.= "<<rand()<<endl; 
        //cout<<"Rand no. gen. for this move= "<<dou_random<<endl;
	if (Probability>dou_random)
	{
	  //accept:
 	  //noccupancy[ijk].site[l].atomtype=noccu_new; //done_before.
	  noccupancy[ijk2].site[l2].atomtype=noccu; //swap on site2.
 	  //cout<<"Rand no. gen. for this move= "<<dou_random<<endl;
	  //cout<<"Move accepted with P= "<<Probability<<endl;
	  //E_struct.dEacc=E_struct.dEacc+dE;
	}
	else
	{
	  //reject:
	  noccupancy[ijk].site[l].atomtype = noccu; //unswap site1.
	  dE=0;
 	  //cout<<"Rand no. gen. for this move= "<<dou_random<<endl;
	  //cout<<"Move rejected with P= "<<Probability<<endl;
	}
      }
      //End of accept or reject portion----------------------------
	E_struct.dEpass+= dE;//dEpass denotes change in energy with respect to Estart.
      
      //if no. of passes >= no. of passes for equilibration
      /*if (pass>equil_pass)
      {
	cout<<"dE= "<<dE<<endl;
	cout<<"noccupancy: "<<endl;
        for (int ijk=0; ijk<nsize; ijk++)
        { //Loop over sublattice
          for (int l=0; l<num_sublat;l++)
          {
            cout<<" "<<noccupancy[ijk].site[l].atomtype;
          }
          for (int i=ndim-1;i>0;i--)
          {
            if ((ijk+1)%factor[i]==0) { cout<<endl; }
       
          }
	}//end for (int ijk=0; ijk<nsize; ijk++)
	}//end_if (pass>equil_pass)*/
    }//end for (int l=0; l<num_sublat;l++)
  }//end (int ijk=0; ijk<nsize;ijk++)
}

//EOF


