//This file is read_vin.cc
//Function definition of member function read_Vin() of class Lattice
//By Teck Leong Tan

#include "lattice.h"

//This function reads in the interactions

void Lattice::read_Vin()
{
  fstream potfile;
  potfile.open(inpotfile, ios::in);
  if (!potfile)
  {
    cout<<"File: "<<inpotfile<<" is not opened!"<<endl;
    exit(1);
  }
  int sub, nn, ff;
  double pot;
  while (1)
  {
    potfile.ignore(256, '\n');//Ignore 1st line of pot.in
    potfile>>sub;//Read in sublattice index
    if (potfile.eof()) {break;} //Break when EOF
    potfile>>nn>>ff;//Read in no. of interacting body and cluster index
    
    if (nn>maxbody)
    {
      cout<<"Warning in read_Vin.cc: No. of potentials given does not match no.";
      cout<<" of interacting bodies in input file"<<endl;
      break;
    }
    
    //Loop over num_type^nn
    for (int i=0; i<power(num_type,nn);i++)
    {
      potfile>>pot;
      clustree.sublat[sub].nbody[nn].nclus[ff]->Vin.push_back(pot);
    }
  }
  potfile.close();
  cout<<"read_vin.cc line 35: Done with read potential file"<<endl;

  //Some checking
  //If zero body Vin is not defined, set it to zero.
  if (clustree.sublat[0].nbody[0].nclus[0]->Vin.empty())
  {
      clustree.sublat[0].nbody[0].nclus[0]->Vin.push_back(0);
  }
 
  if (clustree.sublat[0].nbody[maxbody].nclus[0]->Vin.empty())
  {
    cout<<"Error in read_Vin.cc: No. of potentials given does not match no. ";
    cout<<"of interacting bodies in input file"<<endl;
    exit(1);
  }
     
  //====================================================================
  //The following ensures that all sublattices contains interaction 
  //potentials, by looking at the size of ...sublat[]...Vin[1].
  //If size=0, assume symmetery and copy ...sublat[0]...Vin[] to 
  //...sublat[empty Vin]...Vin[];
  //===================================================================

  //Loop over sublattice
  for (int l=1; l<num_sublat; l++)
  {
    if (clustree.sublat[l].nbody[1].nclus[0]->Vin.size() ==0)
    {
      //Loop over nbody
      for (int n=0; n<maxbody+1;n++)
      {
        for (int f=0; f<clustree.sublat[l].nbody[n].nclus.size(); f++)
        {
          //Check that the size of ...Vin is zero;
          if (clustree.sublat[l].nbody[n].nclus[f]->Vin.size()>0)
          {
            cout<<"read_Vin.cc line56: Error in input file pot.in"<<endl;
            exit(1);
          }  
          for (int i=0; i<clustree.sublat[0].nbody[n].nclus[f]->Vin.size();i++)
          {
            double temp;
            temp=clustree.sublat[0].nbody[n].nclus[f]->Vin[i];
            clustree.sublat[l].nbody[n].nclus[f]->Vin.push_back(temp);
          }
        }
      }//end for (int n=0; n<maxbody;n++)
    }
  }//end for (int l=1; l<num_sublat; l++)  
  //End of checking==================================================
  //cout<<"Out of read_vin.cc"<<endl;
}

//EOF
