//This file is sinversion.cc
//Member function of Lattice class, which performs structural inversion.
//By Teck Leong Tan

#include "lattice.h"
#define ACCURACY 1.0e-2

void get_posXYZ(vector<double> &nei_position, const vector<int>&rposition, const vector< vector<double> >& lat_vec);
void add_vector(vector<double> &a, const vector<double>& b, double constant);
void subtract_vector(vector<double> &a, const vector<double>& b, double constant);
void multiply_const(vector<double> &a, double constant);
//void bound(vector<double> &nei_position,const vector< vector<double> >&inverse, const vector< vector<double> >&matrix, double accuracy);
void bound (vector<double> &nei_position, const Trans_vectors &Tv, const double accuracy);
double dot_vector(const vector<double> &a, const vector<double>& b);
bool equal(const vector<double> &a, const vector<double>& b, const double accuracy);
template<class Etype> void swap(Etype array[], int i, int j);
void Lattice::sinversion()
{
  char e_per_atom='a';//Set to 'a' for energy per atom and 'c' for per cell
  //string structfile="/structure.txt";//Name of file holding structure data 
  vector<string> structdir;//Directories holding structfile
  Inversion Inv(num_type);//An instance of the class Inversion
  int vsize=0;//No. of independent interaction potentials
  int tracker=0;//Keeps track of no. of energy data points
  int num_eachtype [num_type+1]; //No. of atoms for all atom types
  string name_eachtype[num_type+1];//Name of each atom type
  //string orig_sequence[num_type+1];//Sequence of element name as in structure file
  int total_atoms;//Total no. of atoms for structure
  int tempint;
  //vector< vector<double> > struct_vec;//Stores structural vectors
  //vector< vector<double> > struct_vec_inv;//Stores inverse matrix of structural vectors
  //vector<double> struct_vec_mag2;//Stores square of magnitude of struct_vec
  Trans_vectors struct_vec;//Structural vectors
  //double scale;//Stores the scaling factor of the structure vectors
  vector<double> tempvec;  
  string tempstr;
  bool match=0;
  double dist2_accuracy= 1000000;//Accuracy used when comparing square of distance. 
                                //It is usually a fraction of nearest neighbor distance
  double dist_accuracy;//Square root of dist2_accuracy				
  vector<double> minconc,maxconc;
  //Minimum and maximum concentrations of the given structures for each type
  //This is used to determine if the constant term should be used.
  //The constant interaction term is used in grand canoical ensemble
  //but not in canoical ensemble
  minconc.assign(num_type,2); maxconc.assign(num_type,-1);
  tempvec.assign(ndim,0);

  Lat_vec.read_lat_vec(sinv_dir, ndim);//Read in lattice vectors
  //The following calculates dist_accuracy
  //Loop over no. of sublattice
  for (int i=0;i<Lat_vec.basis.size();i++)
  { 
    double dotp;
    //tempvec.assign(ndim,0);
    //Return relative position of neighbor atom in cartesian_XYZ
    get_posXYZ(tempvec, neighborlist[i][1].rposition,Lat_vec.vecs);
    add_vector(tempvec, Lat_vec.basis[0],1);//Add position of sublattice
    dotp=dot_vector(tempvec,tempvec);
    if (dotp <dist2_accuracy){dist2_accuracy=dotp;}
  }
  dist2_accuracy=fracnn2*dist2_accuracy;
  dist_accuracy=sqrt(dist2_accuracy);
  //cout<<"dist2_accuracy= "<<dist2_accuracy<<endl; 
  
  //Allocate size of interaction potentials Vin
  vsize=prep_Vin(Inv);//No. of independent interaction potentials
  Inv.J.assign(vsize,0);

  //Read structure and energy data from files and insert into Inv=======================================================

  //Open file with list of directories where structures are stored
  fstream indir;
  if (!stdata_dir.empty()) {tempstr=stdata_dir+"/"+indirsfile;}
  else {tempstr=indirsfile;}//indirsfile assigned in filenames.h
  indir.open(tempstr.c_str(),ios::in);
  if (!indir)
  {
    cout<<"Error in sinversion.cc: file "<<tempstr<<" not opened!"<<endl;
    exit(1);
  }
  string astring;
  while(getline(indir, astring))
  {
    if (astring[0]!='#')
    {
      if (!stdata_dir.empty()) 
      {
        tempstr=stdata_dir+"/"+astring+structfile;
      }
      else {tempstr=astring+structfile;}
      structdir.push_back(tempstr);
    }  
  }
  indir.close();

  //Loop over number of structure files
  for (int i=0; i<structdir.size();i++) 
  {
    fstream in_struct;
    in_struct.open(structdir[i].c_str(), ios::in);
    if (!in_struct)
    {
      cout<<"File: "<<structdir[i]<<" not opened!"<<endl;
      exit(1);
    }
    
    //The following will read each structure file
    string line;
    //char* temp;
    string sid;
    int iserror=0;
    //vector< vector<double> > pos;//positions of atoms in cartesian_XYZ format
    vector<int> pos_type;//Type of atom occupying position
    vector<int>subin;//Sublattice index of atom at position XYZ
    vector<int> subin_check;//Structure can be mapped into given lattice if all sublattice 
                            //of the lattice occur equal no. of times
    subin_check.assign(Lat_vec.basis.size(),0);
			    
    int corr[vsize];//Correlation terms
    int cor_degen[vsize];//Degeneracies of the corresponding correlation term

    double edge_ratio;//Ratio of an edge of the primitive cell in lat_vec to that of struct_vec 
	
    while ( getline(in_struct, line) )
    {
      //in_struct.ignore(1024,'\n');
      if (in_struct.fail())
      { 
        cerr<<"Error in reading  "<<structdir[i]<<endl;
        exit(1);
      }
      //cout<<"line 51 *sizeof struct_vec= "<<struct_vec.size()<<endl;
      /*cout<<"temp= "<<temp<<endl;
      cout<<strcmp(temp, "%")<<endl;
      cout<<"strncmp(temp, %A,2) is "<<strncmp(temp, "%A",2)<<endl;
      cout<<strncmp(temp, "%E",2)<<endl;*/
      //cout<<"line is "<<line<<endl;
      if (line=="%") 
      {
        //endfile++;
        //in_struct.ignore(1024,'\n');
        in_struct>>sid;
        in_struct.ignore(1024,'\n');
	in_struct.ignore(1024,'\n');
        //in_struct>>scale;      
	//in_struct.ignore(1024,'\n');
	//cout<<"sid= "<<sid<<" scale= "<<scale<<endl;
	if (in_struct.fail())
	{ 
	  cerr<<"Error in reading file  "<<structdir[i]<<endl;
	  exit(1);
        }
        
        //Read structural vectors
	struct_vec.read_struct_vec(in_struct,ndim,Inv);
	cout<<"vol. of sid "<<sid<<" = "<<struct_vec.volume<<endl;
	/*edge_ratio=pow((Lat_vec.volume/Lat_vec.basis.size()/struct_vec.volume/ \
	               struct_vec.basis.size()),0.3333333333333333);
        cout<<"edge_ratio= "<<edge_ratio<<endl;*/		       
	//The above is based on the assumption of cubic unit cell.
	
        /*for (int j=0; j<ndim; j++)
        {
          for(int k=0; k<ndim; k++)
          {
            in_struct>>struct_vec[j][k];
            struct_vec[j][k]*=scale;
            struct_vec_inv[j][k]=struct_vec[j][k];
            struct_vec_mag2[j]+=(struct_vec[j][k]*struct_vec[j][k]);
            cout<<struct_vec[j][k]<<" ";
          }
          //struct_vec_mag[j]=sqrt(struct_vec_mag[j]);
          cout<<" mag2= "<<struct_vec_mag2[j];
          cout<<endl;
        }//end for (int j=0; j<num_type; j++)
        cout<<"end read struct_vec"<<endl;
	in_struct.ignore(1024,'\n');
        Inv.invert_matrix(struct_vec_inv);//Invert the struct_vec matrix*/
	
      }//end if (strncmp(temp,"%",1)==0 && endfile<1)
      
      //cout<<"strncmp(temp, "%A",2) is "<<strncmp(temp, "%A",2)<<endl;
      else if (line=="%A")
      {
        total_atoms=0;
        //in_struct.ignore(1024,'\n');
	//Read in no. of atomic species
        in_struct>>tempint;	
	//cout<<"tempint= "<<tempint<<endl;
	//cout<<"num_type= "<<num_type<<endl;
	if(tempint!=num_type+1)
	{
	  cout<<"Warning in sinversion.cc: No.of atomic species in "<<structdir[i];
	  cout<<" does not match that in input.txt  Structure skipped"<<endl;
	  iserror++;
        }
	  
        //Read number of atoms for each species--------------------------------------------------
	in_struct.ignore(1024,'\n');
	if (i==0)//if it's the 1st structure, read then associate elements to indices
	{
	  string atom_index[num_type+1];
	  for (int j=0; j<num_type+1; j++)
	  {
	    in_struct>>name_eachtype[j]>>num_eachtype[j];
	    atom_index[j]=name_eachtype[j];
	    total_atoms+=num_eachtype[j];
	    if (atom_index[j]>atom_index[0])
	    {
	      //If element is majority species, give it index=0
	      swap(atom_index,0, j);
	     
            } 
	  }//end for (int j=0; j<num_type+1; j++)
	  
	  //Change the assignment of indices to element with user input if any
          for (int j=0; j<elem_index.size(); j++)
	  {
	    for (int kk=0;kk<sizeof(num_eachtype)/sizeof(int);kk++)
	    {
	      if (atom_index[kk]==elem_index[j].element) 
	      {
	        //if index is not as defined by user, swap
	        if(elem_index[j].index != kk && elem_index[j].index<=sizeof(num_eachtype)/sizeof(int))
	        {
	          swap(atom_index,kk,elem_index[j].index);
		}
		else if (elem_index[j].index == kk){;}
	       
                else
	        {
	          cout<<"Warning from sinversion.cc: Element "<<elem_index[j].element;
		  cout<<" is not given the index of "<<elem_index[j].index<<endl;
	        } 
		break;
	      }
	    }//end for (int kk=0;kk<name_eachtype.size()
	  }//End for (int j=0; j<elem_index.size(); j++)  

	  //Finally, update elem_index of class Lattice with latest indices
	  elem_index.clear();
          for (int j=0;j<sizeof(num_eachtype)/sizeof(int);j++)
	  {
	    Element temp;
	    temp.index=j;
	    temp.element=atom_index[j];
	    elem_index.push_back(temp);
	  }  
	} //End if (i==0)

	else //if it's not the 1st structure to be read
        {
	  for (int j=0;j<sizeof(num_eachtype)/sizeof(int);j++)
	  {
	    in_struct>>name_eachtype[j]>>num_eachtype[j];
	    total_atoms+=num_eachtype[j];
	    /*for (int kk=0;kk<elem_index.size();kk++)
	    {
	      if(tempstr==){num_eachtype[kk]=tempint; break;}
	    }*/  
	  }  
        }//End else

	//End of read number of atoms for each species--------------------------------------------
	
        //Check if total no. of atoms is correct
	in_struct>>tempint;
	if (tempint!=total_atoms)
	{
          cout<<"Warning in sinversion.cc: Total no. of atoms in file ";
	  cout<<structdir[i]<<" is incorrect";
	  iserror++;
	}
        
        edge_ratio=pow(((Lat_vec.volume/(double)Lat_vec.basis.size())\
	                /(struct_vec.volume/(double)total_atoms)),0.3333333333333333);
	cout<<Lat_vec.volume<<" "<<Lat_vec.basis.size()<<" ";
	cout<<struct_vec.volume<<" "<<total_atoms<<endl;
	cout<<Lat_vec.volume/(double)Lat_vec.basis.size()/struct_vec.volume/(double)total_atoms<<endl;
        //cout<<"edge_ratio= "<<edge_ratio<<endl;		       
	//The above is based on the assumption of cubic unit cell.

        //Read atom positions--------------------------------------------------------------------------------
        vector<double> temp_pos;
        temp_pos.assign(ndim,0);
        in_struct.ignore(1024, '\n');
        //Loop over atomic type
        for (int j=0; j<sizeof(num_eachtype)/sizeof(int); j++)
        {
          for (int k=0; k<num_eachtype[j]; k++)
          {
            for (int l=0; l<ndim;l++)
            {
              //in_struct.get(temp,5);
              in_struct>>temp_pos[l];
              //temp_pos[l]*=struct_vec.scale;
              //cout<<"'"<<temp_pos[l];
            }
	    //cout<<endl;
            in_struct.ignore(1024, '\n');
            for (int kk=0;kk<elem_index.size();kk++)
	    {
	      if (elem_index[kk].element==name_eachtype[j])
	      {
	        pos_type.push_back(elem_index[kk].index); //Atomic type
	        break;
	      }  
	    }
            bound(temp_pos,struct_vec,dist_accuracy);
            struct_vec.basis.push_back(temp_pos);//Assign cartesian_XYZ position to atom
           
	    //Assign sublattice index to position.
            match=0;
	    multiply_const(temp_pos, edge_ratio);
	    /*cout<<"before 2nd bound: "<<endl;
	    for (int jjj=0;jjj<ndim;jjj++)
            {
              cout<<temp_pos[jjj]<<" ";
            }
	    cout<<endl;*/
	    bound(temp_pos,Lat_vec,dist_accuracy);
	    /*cout<<"after 2nd bound: "<<endl;
	    for (int jjj=0;jjj<ndim;jjj++)
	    {
	      cout<<temp_pos[jjj]<<" ";
	    }
	    cout<<endl;*/
	    //Loop over no. of sublattices
            for(int ll=0;ll<Lat_vec.basis.size();ll++)
            {
	      //multiply_const(temp_pos, edge_ratio);
	      //bound(temp_pos,Lat_vec.inv_vecs,Lat_vec.vecs);	
	      //cout<<"check: "<<endl;
	      double difference=0;
	      for (int k=0;k<ndim;k++)
	      {
	        difference+=((temp_pos[k]-Lat_vec.basis[ll][k])*(temp_pos[k]-Lat_vec.basis[ll][k]));
              }
	      //cout<<"diff= "<<difference<<endl;
			   
              if (equal(temp_pos, Lat_vec.basis[ll], dist2_accuracy))
              {
                match=1;
		subin.push_back(ll);
		subin_check[ll]++;//No. of occurence of basis (or sublat.) index of Lat_vec
                break;
              }
            }
            if (match==0)
            {
              cout<<"Warning in file sinversion.cc: No match for pos["<<k<<"]."<<endl;
              iserror++;
              //exit(1);
            }
          }//end for (int k=0; k<num_eachtype[j]; k++)
        }//end read position

	//Additional check if structure falls into the given lattice
	for (int sub=1; sub<subin_check.size();sub++)
	{
	  if (subin_check[sub]!=subin_check[0])
	  {
	    iserror++;
	    cout<<"Warning in file sinversion.cc: Not all basis are mapped to equal no. of times."<<endl;
	    exit (1);
	    break;
	  }
	}  
        //End reading positions---------------------------------------------------------------------------------
	//Check positions via printing
	/*cout<<"positions: "<<endl;
	for (int j=0;j<struct_vec.basis.size();j++)
	{
	  for (int k=0;k<struct_vec.basis[j].size();k++)
	  {
	    cout<<struct_vec.basis[j][k]<<" ";       
	  }
	  cout<<pos_type[j];
	  cout<<endl;
	}*/

	//Extracting correlation phi-----------------------------------------------------------------------------
	//Loop over clusters. 
	//int corr[vsize];//Correlation terms
	//int cor_degen[vsize];//Degeneracies of the corresponding correlation term
	for (int jj=1; jj<vsize; jj++)
	{
	  corr[jj]=0;
	  cor_degen[jj]=1;//Set to one, so that if corr[jj] is not updated, corr[jj]/cor_degen[jj] is still 0.
	}

	corr[0]=1; cor_degen[0]=1; //correlation of constant interaction term is 1;

	/*for (int jj=0;jj<vsize;jj++)
	{
	   cout<<"corr["<<jj<<"]=  "<<corr[jj]<<endl;
	   cout<<"cor_degen["<<jj<<"]= "<<cor_degen[jj]<<endl;
	}*/

	//Loop over positions
	for (int j=0;j<struct_vec.basis.size();j++)
	{
	  if(iserror>0){break;}

	  //Loop over nbody
	  for (int n=1;n<maxbody+1;n++)
	  { 
	    //Loop over cluster types
	    for (int f= 0;f<clustree.sublat[subin[j]].nbody[n].nclus.size();f++)
	    {
	      //Loop over degeneracy
	      for(int d=0;d<clustree.sublat[subin[j]].nbody[n].nclus[f]->nsym.size();d++)
	      {
		vector<int> noccu;//Atomic species occupying each cluster site
		noccu.assign(n,0);
		bool iszero=0;
		//Loop over neighbor references
		for (int r=0;r<clustree.sublat[subin[j]].nbody[n].nclus[f]->nsym[d]->ref.size();r++)
		{
		  int ref;
		  ref=clustree.sublat[subin[j]].nbody[n].nclus[f]->nsym[d]->ref[r];//neighbor reference
		  vector<double> nei_position;
		  int nei_subin;//Neighbor sublattice index
		  nei_subin=neighborlist[subin[j]][ref].l;
		  //Return relative position of neighbor atom in cartesian_XYZ
		  get_posXYZ(nei_position, neighborlist[subin[j]][ref].rposition,Lat_vec.vecs);
		  
		  add_vector(nei_position, Lat_vec.basis[nei_subin],1);//Add position of sublattice to obtain real relative position
		  
		  multiply_const(nei_position, 1.0/edge_ratio);
		  add_vector(nei_position, struct_vec.basis[j],1);//Position of neighbor=relative position+position of atom
		  
		  bound(nei_position,struct_vec, dist_accuracy);
		  //The above applies boundary conditions (demarcated by struct_vec)to neighbor position 

		  /*cout<<"bou nfd "<<n<<" "<<f<<" "<<d<<" "<<"pos["<<j<<"]neighbors:  "<<"r= "<<r<<" co-ord= ";
		  for(int ii=0; ii<ndim;ii++)
		  {
		      cout<<nei_position[ii]<<" "; 
		  }
		  cout<<endl;*/
		  
		  //Compare nei_position with pos[1...r] to see which the former fits into
		  match=0;
		  for (int bb=0;bb<struct_vec.basis.size();bb++)//Loop over no. of atoms
		  {
		    if (equal(nei_position,struct_vec.basis[bb],dist2_accuracy))
		    {
		      match=1;
		      noccu[r]=pos_type[bb];//Assign atomic type to noccu
		      break;
		    }
		  }//end for (int bb=0;bb<pos.size();bb++)
		  if (match==0)
		  {
		    cout<<"Error: In file sinversion.cc: No match for nei_position."<<endl;
		    iserror++;
		    //exit(1);
		  }
		  //Assign noccu to the corresponding correlation term
		  if (noccu[r]==0) {iszero=1; break;}
		}//end for (int r=0;r<n;r++)

		if (iszero==0)
		{
		  int index=linear_index(n,noccu);
		  int findex=clustree.sublat[subin[j]].nbody[n].nclus[f]->f_index;//The real f-index
		  index=clustree.sublat[0].nbody[n].all_clus[findex][index];
		  //cout<<"index= "<<index<<endl;
		  corr[index]=corr[index]+1;
		  //cor_degen[index]=(clustree.sublat[subin[j]].nbody[n].nclus[f]->nsym.size())*struct_vec.basis.size();//Indices are rewritten over 
		  cor_degen[index]=struct_vec.basis.size()*n;//Indices are rewritten over
		  
		}    
		//}//end for (int j=0;j<pos.size();j++)  
	      }//end for(int d=0;d<clustree.sublat[subin[j]].nbody[n].nclus[f]->nsym.size();d++)
	    }//end (int f....)
	  }//end for (int n=1;n<maxbody+1;n++)
	}//end for (int j=0;j<pos.size();j++)
	//END of extracting correlation phi--------------------------------------------------------------------
      }//End else if (strncmp(temp, "%A",2)==0)  

      else if (line=="%E")
      {
        //Read energy-------------------------------------------------------------------------------------------
        //in_struct.ignore(1024, '\n');
        double E0, Ef, dE;
        in_struct>>E0>>Ef>>dE;
        if (e_per_atom!='a')
	{
	  E0=E0/struct_vec.basis.size();
	  Ef=Ef/struct_vec.basis.size();
	  dE=dE/struct_vec.basis.size();
	}
        //cout<<"E0= "<<E0<<"Ef= "<<Ef<<"dE= "<<dE<<endl;
        //End reading energy--------------------------------------------------------------------------------------         

        //Now update the matrices and vectors in Inversion class
        if(iserror==0)
        {
	  cout<<"iserror= "<<iserror<<endl;
          Inv.E.push_back(E0);
	  Inv.Ef.push_back(Ef);
	  Inv.dE.push_back(dE);
          cout<<"sid= "<<sid<<" E0= "<<E0<<endl;
          Inv.sid.push_back(sid);
          vector<double> temp;
	  double tempd;
          //cout<<"vsize= "<<vsize<<endl;
          //Loop over number of independent interactions
          for (int jj=0;jj<vsize;jj++)
          {
	    tempd=(double)corr[jj]/(double)cor_degen[jj];
            temp.push_back(tempd);
            //cout<<"corr["<<jj<<"]=  "<<corr[jj]<<endl;
            //cout<<"cor_degen["<<jj<<"]= "<<cor_degen[jj]<<endl;
	    if(jj>=1 && jj<=num_type)//if it's one-body cluster. Index 0 reserved for 0-body clus.
	    {
	      if((tempd-minconc[jj-1]) <0){minconc[jj-1]=tempd;}
	      if((tempd-maxconc[jj-1]) >0){maxconc[jj-1]=tempd;}
	    }  
          }
          Inv.phi.push_back(temp);
	  tracker++;
        }
        else
	{
	  cout<<"Error in sid: "<<sid<<".Structure is not "<<Lat_vec.name; 
	  cout<<" and will not be read."<<endl;
        }
	cout<<"end of %E"<<endl;
	break;
      }//End else if (strncmp(temp, "%E",2)==0)	
    }//end while(.....)
    //cout<<"out of while loop"<<endl;
    in_struct.close();
  }//end for (int i=0; i<structdir.size();i++) 
  //End of reading and inserting into Inv==========================================================================
  cout<<tracker<<" structures out of "<<structdir.size()<<" structures read successfully."<<endl;
  
  //Do structural inversion========================================================================================
  //Determine if the structural data is for grand canonical or canonical ensemble
  cout<<"Interactions will be good for following composition range:"<<endl;
  cout<<"Atom type  Range"<<endl; 
  for (int j=0;j<num_type;j++)
  {
    cout<<setw(9)<<j<<"  ";
    cout<<minconc[j]<<" to "<<maxconc[j]<<endl;
    if(fabs(maxconc[j]-minconc[j])>1.0e-6)
      {Inv.ensemble='g'; break; }
    else 
    {
      Inv.ensemble='c';
      /*cal_corr_disorder(Inv);//Calculates the disorder energy
      cout<<"check disorder:"<<endl;
      for (int dis=0; dis<Inv.corr_disorder.size();dis++)
      {
        cout<<Inv.corr_disorder[dis]<<endl;
      }
      cout<<"Edisorder= "<<Inv.Edisorder<<endl;*/
      
    }
  } 
  //Inv.select_clus(); 
  Inv.set_param();
  Inv.print_outphi();
  Inv.lfit(); //Structural inversion with general least squares.
  cout<<"lfit() done"<<endl;
  Inv.cv1_fast();
  //Calculate disorder energy if canoical
  if (Inv.ensemble=='c')
  {
      cal_corr_disorder(Inv);//Calculates the disorder energy      
      //cout<<"check disorder:"<<endl;
      /*for (int dis=0; dis<Inv.corr_disorder.size();dis++)
      {
        cout<<Inv.corr_disorder[dis]<<endl;
      }*/
      cout<<"Edisorder= "<<Inv.Edisorder<<endl;
  }
  cout<<"cv1_fast() done"<<endl;
  Inv.print_matrix("out_struct.txt");
  create_Vin(Inv.J);//Transfer interactions to Vin vectors in class Ncluster and print to file.
  //END of structural inversion=====================================================================================
}


void get_posXYZ(vector<double> &nei_position, const vector<int> &rposition, const vector< vector<double> >& lat_vec)
{
  //The func return relative position of neighbor atom in cartesian_XYZ 
  //through nei_position
  int dimension=rposition.size();
  nei_position.assign(dimension,0);
  for(int i=0; i<dimension;i++)
  {
    double temp=0;
    for (int j=0;j<dimension;j++)
    {
      temp+=((double)rposition[j]*lat_vec[j][i]);
      //cout<<"rposition["<<i<<"] * lat_vec["<<j<<"]["<<i<<"]= "<<rposition[i]<<"*"<<lat_vec[j][i];
    }
    //cout<<endl;
    nei_position[i]=temp;
  }
}

void add_vector(vector<double> &a, const vector<double>& b, const double constant)
{
  //Addition of vectors with the new vector stored back into a
  int dimension=b.size();
  if (dimension!=a.size())
  {cerr<<"Error in sinversion.cc: Vector sizes must be the same for addition"<<endl; exit(1);}
  for (int j=0;j<dimension;j++)
  {
      a[j]=a[j]+(constant*b[j]);
  }
}

void subtract_vector(vector<double> &a, const vector<double>& b, const double constant)
{
  //Subtraction of vectors with the new vector stored back into a
  int dimension=b.size();
  if (dimension!=a.size())
  {cerr<<"Error in sinversion.cc: Vector sizes must be the same for subtraction"<<endl;exit(1);}
  for (int j=0;j<dimension;j++)
  {
      a[j]=a[j]-(constant*b[j]);
  }
}

void multiply_const(vector<double> &a, double constant)
{
  for (int j=0;j<a.size();j++)
  {
    a[j]=a[j]*constant;
  }
}	      

void bound (vector<double> &nei_position, const Trans_vectors &Tv, const double cart_accuracy)
{
  int dimension=nei_position.size();//In cartesian_XYZ
  vector<double> pos_ijk;//In cartesian_IJK, where I, J, K are vectors corresponding to matrix
  //Matrix is dimension * dimension
  pos_ijk.assign(dimension,0);
  //cout<<"in ijk: ";
  for(int i=0;i<dimension;i++)
  {
    for(int j=0;j<dimension;j++)
    {
      pos_ijk[i]+=(nei_position[j]*Tv.inv_vecs[j][i]);
      //cout<<matrix[i][j]<<" ";
    }
    //cout<<nei_position[i]<<" ";
    //cout<<" ** "<<pos_ijk[i]<<" **  ";
    
  }
  
  //cout<<endl;
  //cout<<"during bc: "<<endl; 
  //cout<<"accuracy= "<<cart_accuracy<<"/"<<sqrt(Tv.mag2[0])<<" = "<<cart_accuracy/sqrt(Tv.mag2[0]);
  for (int i=0;i<dimension;i++)
  {
    int temp=(int)pos_ijk[i];
    //cout<<nei_position[i]<<" "; 
    //cout<<"temp= "<<temp<<endl;
    subtract_vector(nei_position, Tv.vecs[i],temp);
    pos_ijk[i]-=temp;
    //cout<<" ** "<<pos_ijk[i]<<" **  ";
    //if IJK_position < -IJK_accuracy
    if (pos_ijk[i]<(-(cart_accuracy/sqrt(Tv.mag2[i]))))//Need to convert to IJK accuracy
    {
      add_vector(nei_position,Tv.vecs[i],1);
    }
    //if IJK_position>IJK_accuracy
    else if (pos_ijk[i]>(1-(cart_accuracy/sqrt(Tv.mag2[i]))))
    {
      subtract_vector(nei_position,Tv.vecs[i], 1);
    }
  }
  //cout<<endl;
 
}

double dot_vector(const vector<double> &a, const vector<double>& b)
{
  int dim =a.size();
  if (dim!=b.size())
  {cerr<<"Error in sinversion.cc: Vector sizes must be the same for dot product"<<endl; exit(1);}
  double dot_prod=0;
  for (int k=0;k<dim;k++)
  {
    dot_prod+=(a[k]*b[k]);
  }
  return dot_prod;
}

bool equal(const vector<double> &a, const vector<double>& b, const double accuracy)
{
  //int count=0;
  double difference=0;
  int dim=a.size();
  if (dim!=b.size())
  {cerr<<"Error in sinversion.cc: Vector sizes must be the same for comparision"<<endl; exit(1);}
  for (int k=0;k<dim;k++)
  {
    difference+=((a[k]-b[k])*(a[k]-b[k]));
  }
  //cout<<"difference= "<<difference<<endl;
  if (difference<accuracy){return 1;}
  else {return 0;}
  
  
}

template<class Etype> void swap(Etype array[], int i, int j)
{
  Etype temp=array[j];              
  array[j]=array[i];
  array[i]=temp;
}  
 
//EOF
